//
//  GameView.swift
//  Quack
//
//  Created by Marco Fattorel on 20/01/2017.
//  Copyright (c) 2017 Marco Fattorel. All rights reserved.
//

import SceneKit
import SpriteKit

class GameView: SCNView {
	
	private var currentOverlay:(()->Void)? = nil
	// MARK: 2D Overlay
	let overlayNode = SKNode()
	private let collectedCrumbsCountLabel = SKLabelNode(fontNamed: "Chalkduster")
	
	#if os(iOS) || os(tvOS)
	
	override func awakeFromNib() {
	super.awakeFromNib()
	setup2DOverlay()
	}
	
	override func layoutSubviews() {
	super.layoutSubviews()
	layout2DOverlay()
	}
	
	#elseif os(OSX)
	
	override func viewDidMoveToWindow() {
		super.viewDidMoveToWindow()
		setup2DOverlay()
	}
	
	override func setFrameSize(_ newSize: NSSize) {
		super.setFrameSize(newSize)
		layout2DOverlay()
	}
	
	#endif
	
	private func layout2DOverlay() {
		overlayNode.position = CGPoint(x: 0.0, y: bounds.size.height)
		currentOverlay?()
	}
	
	private func setup2DOverlay() {
		let w = bounds.size.width
		let h = bounds.size.height
		
		let skScene = SKScene(size: CGSize(width: w, height: h))
		skScene.scaleMode = .resizeFill
		
		skScene.addChild(overlayNode)
		overlayNode.position = CGPoint(x: 0.0, y: bounds.size.height)
		
//		collectedCrumbsCountLabel.text = "x0"
//		collectedCrumbsCountLabel.position = CGPoint(x: 40, y: -40)
//		overlayNode.addChild(collectedCrumbsCountLabel)
		
		overlaySKScene = skScene
		skScene.isUserInteractionEnabled = false
	}
	
	var collectedCrumbsCount = 0 {
		didSet {
			collectedCrumbsCountLabel.text = "x\(collectedCrumbsCount)"
		}
	}
	
    override func mouseDown(with theEvent: NSEvent) {
        /* Called when a mouse click occurs */
        
        // check what nodes are clicked
        let p = self.convert(theEvent.locationInWindow, from: nil)
        let hitResults = self.hitTest(p, options: [:])
        // check that we clicked on at least one object
        if hitResults.count > 0 {
            // retrieved the first clicked object
            let result = hitResults.first!.node
			
			if result.name == "good" {
				collectedCrumbsCount += 1
				result.removeFromParentNode()
//				createExplosion(geometry: result.geometry!, position: result.presentation.position, rotation: result.presentation.rotation)
			}
			else if result.name == "bad" {
				collectedCrumbsCount -= 1
				result.removeFromParentNode()
//				createExplosion(geometry: result.geometry!, position: result.presentation.position, rotation: result.presentation.rotation)
			}
            
            // get its material
//            let material = result.node!.geometry!.firstMaterial!
//            
//            // highlight it
//            SCNTransaction.begin()
//            SCNTransaction.animationDuration = 0.5
//            
//            // on completion - unhighlight
//            SCNTransaction.completionBlock = {
//                SCNTransaction.begin()
//                SCNTransaction.animationDuration = 0.5
//                
//                material.emission.contents = NSColor.black
//                
//                SCNTransaction.commit()
//            }
//            
//            material.emission.contents = NSColor.red
//            
//            SCNTransaction.commit()
        }
        
        super.mouseDown(with: theEvent)
    }
	
	var xStatusSetup = [Bool]() {
		didSet {
			for (index, status) in xStatusSetup.enumerated() {
				switch index {
				case 0: xStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/blue_f.png") : SKTexture(imageNamed: "Asset.scnassets/blue_e.png")
				case 1: xStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/rose_f.png") : SKTexture(imageNamed: "Asset.scnassets/rose_e.png")
				case 2: xStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/green_f.png") : SKTexture(imageNamed: "Asset.scnassets/green_e.png")
				case 3: xStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/gray_f.png") : SKTexture(imageNamed: "Asset.scnassets/gray_e.png")
				default: break
				}
			}
		}
	}
	// grigio verde rosa azzurro
	var gStatusSetup = [Bool]() {
		didSet {
			for (index, status) in gStatusSetup.enumerated() {
				switch index {
				case 0: gStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/gray_f.png") : SKTexture(imageNamed: "Asset.scnassets/gray_e.png")
				case 1: gStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/green_f.png") : SKTexture(imageNamed: "Asset.scnassets/green_e.png")
				case 2: gStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/rose_f.png") : SKTexture(imageNamed: "Asset.scnassets/rose_e.png")
				case 3: gStatusSetupSprites[index].texture = status ? SKTexture(imageNamed: "Asset.scnassets/blue_f.png") : SKTexture(imageNamed: "Asset.scnassets/blue_e.png")
				default: break
				}
			}
		}
	}
	private var xStatusSetupSprites = [SKSpriteNode]()
	private var gStatusSetupSprites = [SKSpriteNode]()
	
	func showSetupScreen() {
		currentOverlay = showSetupScreen
		overlayNode.removeAllChildren()
		
		
		let setupScreen = SKNode()
		
		let title = SKSpriteNode(imageNamed: "title_center_28.png", position: CGPoint(x: bounds.size.width/2, y: -28), scale: 1.0)
		title.anchorPoint = CGPoint(x: 0.5, y: 1.0)
		
		let credits = SKSpriteNode(imageNamed: "credits.png", position: CGPoint(x: bounds.size.width/2, y: -240), scale: 1.0)
		credits.anchorPoint = CGPoint(x: 0.5, y: 1.0)
		
		let raggiX = SKSpriteNode(imageNamed: "raggi_x_315_578.png", position: CGPoint(x: 315, y: -578), scale: 1.0)
		raggiX.anchorPoint = CGPoint(x: 0.0, y: 0.0)
		
		let raggiG = SKSpriteNode(imageNamed: "raggi_g_792_578.png", position: CGPoint(x: 792, y: -578), scale: 1.0)
		raggiG.anchorPoint = CGPoint(x: 0.0, y: 0.0)
		
		let moveSX = SKSpriteNode(imageNamed: "movesx_42_578.png", position: CGPoint(x: 42, y: -578), scale: 1.0)
		moveSX.anchorPoint = CGPoint(x: 0.0, y: 0.0)
		
		let moveDX = SKSpriteNode(imageNamed: "movedx_966_578.png", position: CGPoint(x: 966, y: -578), scale: 1.0)
		moveDX.anchorPoint = CGPoint(x: 0.0, y: 0.0)
		
		let xDescription = SKSpriteNode(imageNamed: "text_1.png", position: CGPoint(x: raggiX.position.x+84, y: raggiX.position.y-22), scale: 1.0)
		xDescription.anchorPoint = CGPoint(x: 0.5, y: 1.0)
		
		let gDescription = SKSpriteNode(imageNamed: "text_2.png", position: CGPoint(x: raggiG.position.x+84, y: raggiG.position.y-22), scale: 1.0)
		gDescription.anchorPoint = CGPoint(x: 0.5, y: 1.0)
		
		if(xStatusSetup.count == 0 && gStatusSetup.count == 0) {
			xStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiX.position.x, y: xDescription.position.y - xDescription.size.height - 40), scale: 1.0))
			xStatusSetup.append(false)
			xStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiX.position.x + 86, y: xDescription.position.y - xDescription.size.height - 40), scale: 1.0))
			xStatusSetup.append(false)
			xStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiX.position.x + (86*2), y: xDescription.position.y - xDescription.size.height - 40), scale: 1.0))
			xStatusSetup.append(false)
			xStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiX.position.x + (86*3), y: xDescription.position.y - xDescription.size.height - 40), scale: 1.0))
			xStatusSetup.append(false)
			
			gStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiG.position.x, y: gDescription.position.y - gDescription.size.height - 40), scale: 1.0))
			gStatusSetup.append(false)
			gStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiG.position.x + 86, y: gDescription.position.y - gDescription.size.height - 40), scale: 1.0))
			gStatusSetup.append(false)
			gStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiG.position.x + (86*2), y: gDescription.position.y - gDescription.size.height - 40), scale: 1.0))
			gStatusSetup.append(false)
			gStatusSetupSprites.append(SKSpriteNode(imageNamed: "blue_e.png", position: CGPoint(x: raggiG.position.x + (86*3), y: gDescription.position.y - gDescription.size.height - 40), scale: 1.0))
			gStatusSetup.append(false)
		}
		
		
//		collectedCrumbsCountLabel.text = "Group A press MOVE_Button"
//		collectedCrumbsCountLabel.horizontalAlignmentMode = .left
//		collectedCrumbsCountLabel.position = CGPoint(x: 40, y: -40)
//		collectedCrumbsCountLabel.fontSize
//		setupScreen.addChild(collectedCrumbsCountLabel)
		setupScreen.addChild(title)
		setupScreen.addChild(credits)
		setupScreen.addChild(raggiX)
		setupScreen.addChild(raggiG)
		setupScreen.addChild(moveSX)
		setupScreen.addChild(moveDX)
		setupScreen.addChild(xDescription)
		setupScreen.addChild(gDescription)
		
		for xSprites in xStatusSetupSprites {
			setupScreen.addChild(xSprites)
		}
		for gSprites in gStatusSetupSprites {
			setupScreen.addChild(gSprites)
		}
		
		overlayNode.addChild(setupScreen)
	}
	
	var introFrame = 0 {
		didSet {
			switch introFrame {
			case 0: introSprite.texture = SKTexture(imageNamed: "Asset.scnassets/STORY/Intro/Intro_01.png")
			case 1: introSprite.texture = SKTexture(imageNamed: "Asset.scnassets/STORY/Intro/Intro_02.png")
			case 2: introSprite.texture = SKTexture(imageNamed: "Asset.scnassets/STORY/Intro/Intro_03.png")
			case 3: introSprite.texture = SKTexture(imageNamed: "Asset.scnassets/STORY/Intro/Intro_04.png")
			default: break
			}
		}
	}
	private var introSprite: SKSpriteNode = SKSpriteNode()

	func showIntro() {
		currentOverlay = showIntro
		overlayNode.removeAllChildren()
		
		introSprite = SKSpriteNode(imageNamed: "STORY/Intro/Intro_01.png", position: CGPoint(x: bounds.size.width/2, y: -bounds.size.height/2), scale: 1.0)
		overlayNode.addChild(introSprite)
		
//		let stronzo = SKSpriteNode(imageNamed: "angryG.png")
//		stronzo.position = CGPoint(x: 280, y: -400)
//		overlayNode.addChild(stronzo)
		//pointXLabel.text = " ciaone "

	}
	
	private let icsLife = SKLabelNode(fontNamed: "Chalkduster")
	private let gammaLife = SKLabelNode(fontNamed: "Chalkduster")

	private let actionLabel = SKLabelNode(fontNamed: "Chalkduster")
	var actionName = "aaaaa" {
		didSet {
			actionLabel.text = actionName
		}
	}
	
	private let pointXLabel = SKLabelNode(fontNamed: "Chalkduster")
	var pointX = 0 {
		didSet {
			pointXLabel.text = "\(pointX)"
		}
	}
	private let pointGLabel = SKLabelNode(fontNamed: "Chalkduster")
	var pointG = 0 {
		didSet {
			pointGLabel.text = "\(pointG)"
		}
	}
	
	private let ics = SKNode()
	var icsPosition = CGPoint(x: 0, y: -226) {
		didSet {
			ics.position = icsPosition
		}
	}
	
	private let gamma = SKNode()
	var gammaPosition = CGPoint(x: 1280, y: -570) {
		didSet {
			gamma.position = gammaPosition
		}
	}
	
	func showPhaseOne() {
		currentOverlay = showPhaseOne
		overlayNode.removeAllChildren()
		
		icsLife.text = "Vite di Ics"
		icsLife.fontColor = NSColor(calibratedRed: 230, green: 255, blue: 0, alpha: 1)
		icsLife.position = CGPoint(x: bounds.size.width/2 - 300, y: -bounds.size.height/2)
		icsLife.horizontalAlignmentMode = .right
		icsLife.verticalAlignmentMode = .center
		
		gammaLife.text = "Vite di Gamma"
		gammaLife.fontColor = NSColor(calibratedRed: 0, green: 220, blue: 0, alpha: 1)
		gammaLife.position = CGPoint(x: bounds.size.width/2 + 300, y: -bounds.size.height/2)
		gammaLife.horizontalAlignmentMode = .left
		gammaLife.verticalAlignmentMode = .center
		
		actionLabel.position = CGPoint(x: bounds.size.width/2, y: -bounds.size.height/2)
		actionLabel.horizontalAlignmentMode = .center
		actionLabel.verticalAlignmentMode = .center
		
		pointXLabel.position = CGPoint(x: bounds.size.width/2 - 250, y: -bounds.size.height/2)
		pointXLabel.horizontalAlignmentMode = .center
		pointXLabel.verticalAlignmentMode = .center
		
		pointGLabel.position = CGPoint(x: bounds.size.width/2 + 250, y: -bounds.size.height/2)
		pointGLabel.horizontalAlignmentMode = .center
		pointGLabel.verticalAlignmentMode = .center
		
		let codaEmitterIcs = SKEmitterNode(fileNamed: "Asset.scnassets/codaIcs.sks")!
		let codaEmitterGamma = SKEmitterNode(fileNamed: "Asset.scnassets/codaGamma.sks")!
		codaEmitterIcs.particleTexture = SKTexture(imageNamed: "Asset.scnassets/spark.png")
		codaEmitterGamma.particleTexture = SKTexture(imageNamed: "Asset.scnassets/spark.png")
		
		let angryIcs = SKSpriteNode(imageNamed: "neutroX.png",position: CGPoint(x: 0, y: 0), scale: 1.0)
		let angryGamma = SKSpriteNode(imageNamed: "neutroG.png", position: CGPoint(x: 0, y: 0), scale: 1.0)
		
		angryIcs.anchorPoint = CGPoint(x: 0.5, y: 0.5)
		
		angryGamma.anchorPoint = CGPoint(x: 0.5, y: 0.5)
		ics.removeAllChildren()
		ics.addChild(codaEmitterIcs)
		ics.addChild(angryIcs)
		gamma.removeAllChildren()
		gamma.addChild(codaEmitterGamma)
		gamma.addChild(angryGamma)
		
		ics.position = icsPosition
		gamma.position = gammaPosition
		
		overlayNode.addChild(ics)
		overlayNode.addChild(gamma)

		overlayNode.addChild(icsLife)
		overlayNode.addChild(gammaLife)
		
		overlayNode.addChild(pointXLabel)
		overlayNode.addChild(pointGLabel)
		overlayNode.addChild(actionLabel)

	}
	
	func showPhaseFree() {
		currentOverlay = showPhaseFree
		overlayNode.removeAllChildren()
		
		icsLife.text = "Vite di Ics"
		icsLife.fontColor = NSColor(calibratedRed: 230, green: 255, blue: 0, alpha: 1)
		icsLife.position = CGPoint(x: bounds.size.width/2 - 300, y: -100)
		icsLife.horizontalAlignmentMode = .right
		icsLife.verticalAlignmentMode = .center
		
		gammaLife.text = "Vite di Gamma"
		gammaLife.fontColor = NSColor(calibratedRed: 0, green: 220, blue: 0, alpha: 1)
		gammaLife.position = CGPoint(x: bounds.size.width/2 + 300, y: -100)
		gammaLife.horizontalAlignmentMode = .left
		gammaLife.verticalAlignmentMode = .center
		
		pointXLabel.position = CGPoint(x: bounds.size.width/2 - 250, y: -100)
		pointXLabel.horizontalAlignmentMode = .center
		pointXLabel.verticalAlignmentMode = .center
		
		pointGLabel.position = CGPoint(x: bounds.size.width/2 + 250, y: -100)
		pointGLabel.horizontalAlignmentMode = .center
		pointGLabel.verticalAlignmentMode = .center
		
		overlayNode.addChild(icsLife)
		overlayNode.addChild(gammaLife)
		
		overlayNode.addChild(pointXLabel)
		overlayNode.addChild(pointGLabel)
		
		
		let cerchiEmitterIcs = SKEmitterNode(fileNamed: "Asset.scnassets/cerchiIcs.sks")!
		let cerchiEmitterGamma = SKEmitterNode(fileNamed: "Asset.scnassets/cerchiGamma.sks")!
		cerchiEmitterIcs.particleTexture = SKTexture(imageNamed: "Asset.scnassets/spark.png")
		cerchiEmitterGamma.particleTexture = SKTexture(imageNamed: "Asset.scnassets/spark.png")
		
		let angryIcs = SKSpriteNode(imageNamed: "angryX.png",position: CGPoint(x: 0, y: 0), scale: 1.0)
		let angryGamma = SKSpriteNode(imageNamed: "angryG.png", position: CGPoint(x: 0, y: 0), scale: 1.0)
		
		angryIcs.anchorPoint = CGPoint(x: 0.5, y: 0.5)
		icsPosition = CGPoint(x: bounds.size.width/2 - 300, y: -bounds.size.height/2)
		
		angryGamma.anchorPoint = CGPoint(x: 0.5, y: 0.5)
		gammaPosition = CGPoint(x: bounds.size.width/2 + 300, y: -bounds.size.height/2)
		
		ics.removeAllChildren()
		ics.addChild(angryIcs)
		ics.addChild(cerchiEmitterIcs)
		gamma.removeAllChildren()
		gamma.addChild(angryGamma)
		gamma.addChild(cerchiEmitterGamma)
		
		overlayNode.addChild(ics)
		overlayNode.addChild(gamma)
	}
	
	
	var gammaWin = false
	var icsWin = false
	func showPhaseResult() {
		currentOverlay = showPhaseFree
		overlayNode.removeAllChildren()
		
		
		let angryIcs = icsWin ? SKSpriteNode(imageNamed: "feliceX.png",position: CGPoint(x: 0, y: 0), scale: 1.0) : SKSpriteNode(imageNamed: "mortoX.png",position: CGPoint(x: 0, y: 0), scale: 1.0)
		let angryGamma = gammaWin ? SKSpriteNode(imageNamed: "feliceG.png", position: CGPoint(x: 0, y: 0), scale: 1.0) : SKSpriteNode(imageNamed: "mortoG.png", position: CGPoint(x: 0, y: 0), scale: 1.0)
		
		let cerchiEmitterIcs = SKEmitterNode(fileNamed: "Asset.scnassets/cerchiIcs.sks")!
		let cerchiEmitterGamma = SKEmitterNode(fileNamed: "Asset.scnassets/cerchiGamma.sks")!
		cerchiEmitterIcs.particleTexture = SKTexture(imageNamed: "Asset.scnassets/spark.png")
		cerchiEmitterGamma.particleTexture = SKTexture(imageNamed: "Asset.scnassets/spark.png")
		
		angryIcs.anchorPoint = CGPoint(x: 0.5, y: 0.5)
		icsPosition = CGPoint(x: bounds.size.width/2 - 300, y: -bounds.size.height/2)
		
		angryGamma.anchorPoint = CGPoint(x: 0.5, y: 0.5)
		gammaPosition = CGPoint(x: bounds.size.width/2 + 300, y: -bounds.size.height/2)
		
		ics.removeAllChildren()
		ics.addChild(angryIcs)
		if icsWin {
			ics.addChild(cerchiEmitterIcs)
		}
		gamma.removeAllChildren()
		gamma.addChild(angryGamma)
		if gammaWin {
			gamma.addChild(cerchiEmitterGamma)
		}
		
		overlayNode.addChild(ics)
		overlayNode.addChild(gamma)
	}
	
	var endFrame = 0 {
		didSet {
			switch endFrame {
			case 0: endSprite.texture = SKTexture(imageNamed: "Asset.scnassets/STORY/End/End_01.png")
			case 1: endSprite.texture = SKTexture(imageNamed: "Asset.scnassets/STORY/End/End_02.png")
			default: break
			}
		}
	}
	private var endSprite: SKSpriteNode = SKSpriteNode()
	
	func showEnd() {
		currentOverlay = showIntro
		overlayNode.removeAllChildren()
		
		endSprite = SKSpriteNode(imageNamed: "STORY/End/End_01.png", position: CGPoint(x: bounds.size.width/2, y: -bounds.size.height/2), scale: 1.0)
		overlayNode.addChild(endSprite)		
	}
	
	func restart() {
		pointX = 0
		pointG = 0
		gammaWin = false
		icsWin = false
		actionName = ""
		icsPosition = CGPoint(x: 0, y: -226)
		gammaPosition = CGPoint(x: 1280, y: -570)
	}
}
