//
//  GameViewController.swift
//  Quack
//
//  Created by Marco Fattorel on 20/01/2017.
//  Copyright (c) 2017 Marco Fattorel. All rights reserved.
//

import SceneKit
import GameController
import QuartzCore

public extension Float {
	/// Returns a random floating point number between 0.0 and 1.0, inclusive.
	public static var random:Float {
		get {
			return Float(arc4random()) / 0xFFFFFFFF
		}
	}
	/**
	Create a random num Float
	
	- parameter min: Float
	- parameter max: Float
	
	- returns: Float
	*/
	public static func random(min: Float, max: Float) -> Float {
		return Float.random * (max - min) + min
	}
}
func randomColor() -> NSColor {
	let hue = CGFloat(Double(arc4random() % 256) / 256.0) // 0.0 to 1.0
	let saturation = CGFloat(Double(arc4random() % 128) / 266.0 + 0.5) // 0.5 to 1.0, away from white
	let brightness = CGFloat(Double(arc4random() % 128) / 256.0 + 0.5) // 0.5 to 1.0, away from black
	return NSColor(hue: hue, saturation: saturation, brightness: brightness, alpha: 1.0)
}

class GameViewController: NSViewController {
	
    @IBOutlet weak var gameView: GameView!
	var cameraNode: SCNNode!
	var startTime: TimeInterval = 0.0
	
	enum GameStatus {
		case setup
		case intro
		case phaseOne
		case phaseFree
		case phaseResult
		case phaseEnd
		case restart
	}
	var gameStatus = GameStatus.setup
	
    override func awakeFromNib(){
        super.awakeFromNib()
		setupView()
		setupCamera()
//		setupController()
		Gamepad.initialize()
		Move.initialize()
//		let move = Move()
		
//        let scene = SCNScene(named: "art.scnassets/ship.scn")!
	}
	
	func setupView() {
		let scene = SCNScene()
		self.gameView!.scene = scene
		
//		self.gameView!.showsStatistics = true
		self.gameView!.allowsCameraControl = false
		self.gameView!.autoenablesDefaultLighting = true
		self.gameView!.backgroundColor = NSColor(hue: 0, saturation: 0, brightness: 0.30, alpha: 1)
		self.gameView!.delegate = self
		self.gameView!.isPlaying = true
		
		gameStatus = .setup
		gameView.showSetupScreen()
	}
	
	func setupCamera() {
		cameraNode = SCNNode()
		cameraNode.camera = SCNCamera()
		cameraNode.position = SCNVector3(0, 5, 10)
		self.gameView!.scene!.rootNode.addChildNode(cameraNode)
	}
	func setupController() {
//		var pad = Gamepad()
//		let controllers = CCController.controllers()
//		print(controllers.count)
//		
//		let activateController : (GCController) -> Void = { controller in
//			print(controller.vendorName!)
//			var observer: NSObjectProtocol = () as! NSObjectProtocol
//			observer = NotificationCenter.default.addObserver(forName: .GCControllerDidDisconnect, object: controller, queue: nil, using: { notification in
//					print("removed")
//					NotificationCenter.default.removeObserver(observer)
//			})
//		}
//		for controller in controllers {
//			activateController(controller)
//		}
//		NotificationCenter.default.addObserver(forName: .GCControllerDidConnect, object: nil, queue: nil, using: { notification in
//			print("added")
//			activateController(notification.object as! GCController)
//		})
	}
	
//	func spawnShape() {
//		var geometry: SCNGeometry
//		
//		switch ShapeType.random() {
//		case .Box:
//			geometry = SCNBox(width: 1.0, height: 1.0, length: 1.0, chamferRadius: 0.0)
//		case .Capsule:
//			geometry = SCNCapsule(capRadius: 0.25, height: 1.0)
//		case .Cone:
//			geometry = SCNCone(topRadius: 0.0, bottomRadius: 0.5, height: 1.0)
//		case .Cylinder:
//			geometry = SCNCylinder(radius: 0.5, height: 1.0)
//		case .Pyramid:
//			geometry = SCNPyramid(width: 1.0, height: 1.0, length: 1.0)
//		case .Sphere:
//			geometry = SCNSphere(radius: 0.5)
//		case .Torus:
//			geometry = SCNTorus(ringRadius: 0.5, pipeRadius: 0.25)
//		case .Tube:
//			geometry = SCNTube(innerRadius: 0.25, outerRadius: 0.5, height: 1.0)
//		}
//		
//		let color = randomColor()
//		geometry.materials.first?.diffuse.contents = color
//		let geometryNode = SCNNode(geometry: geometry)
//		geometryNode.physicsBody = SCNPhysicsBody(type: .dynamic, shape: nil)
//		let trailEmitter = createTrail(color: color, geometry: geometry)
//		geometryNode.addParticleSystem(trailEmitter)
//		if color == NSColor.black {
//			geometryNode.name = "bad"
//		}
//		else {
//			geometryNode.name = "good"
//		}
//		
//		let randomX = Float.random(min: -2, max: 2)
//		let randomY = Float.random(min: 10, max: 18)
//		let force = SCNVector3(x: CGFloat(randomX), y: CGFloat(randomY), z: 0.0)
//		let position = SCNVector3(x: 0.05, y: 0.05, z: 0.05)
//		geometryNode.physicsBody?.applyForce(force, at: position, asImpulse: true)
//		
//		self.gameView!.scene!.rootNode.addChildNode(geometryNode)
//	}
	
//	func cleanScene() {
//		for node in gameView!.scene!.rootNode.childNodes {
//			if node.presentation.position.y < -2 {
//				node.removeFromParentNode()
//			}
//		}
//	}
	
//	func createTrail(color: NSColor, geometry: SCNGeometry) -> SCNParticleSystem {
//		let trail = SCNParticleSystem(named: "Trail.scnp", inDirectory: "Asset.scnassets")!
//		trail.particleColor = color
//		trail.emitterShape = geometry
//		return trail
//	}
	
//	func createExplosion(geometry: SCNGeometry, position: SCNVector3, rotation: SCNVector4) {
//		let explosion = SCNParticleSystem(named: "Explode.scnp", inDirectory: "Asset.scnassets")!
//		explosion.emitterShape = geometry
//		explosion.birthLocation = .surface
//		
//		let rotationMatrix = SCNMatrix4MakeRotation(rotation.w, rotation.x, rotation.y, rotation.z)
//		let translationMatrix = SCNMatrix4MakeTranslation(position.x, position.y, position.z)
//		let transformMatrix = SCNMatrix4Mult(rotationMatrix, translationMatrix)
//		self.gameView!.scene!.addParticleSystem(explosion, transform: transformMatrix)
//	}
	
	func setupPlayers(_ time: TimeInterval) {
		let movesA = Move.moves.filter{ $0.1.snapshot.buttonMove < 0.5 && $0.1.snapshot.backTrigger > 0.5 }
		let movesB = Move.moves.filter{ $0.1.snapshot.buttonMove > 0.5 && $0.1.snapshot.backTrigger < 0.5 }
//		switch movesB.count {
//		case 3: print("setup complete")
//		case let x where x<3: print("need \(3-x) more player")
//		case let x where x>3: print("need \(x-3) lees player")
//		default: break
//		}
		let padA = Gamepad.controllers.filter{ $0.1.snapshot.buttonA < 0.5 && ($0.1.snapshot.rightTrigger > 0.5 || $0.1.snapshot.leftTrigger > 0.5) }
		let padB = Gamepad.controllers.filter{ $0.1.snapshot.buttonA > 0.5 && $0.1.snapshot.rightTrigger < 0.5 && $0.1.snapshot.leftTrigger < 0.5 }
		
		let sc = gameView
		if padA.count == 1 {
			sc?.xStatusSetup[3] = true
		} else {
			sc?.xStatusSetup[3] = false
		}
		
		if padB.count == 1 {
			sc?.gStatusSetup[0] = true
		} else {
			sc?.gStatusSetup[0] = false
		}
		
		for i in 0..<3 {
			sc?.xStatusSetup[i] = false
		}
		if movesA.count <= 3 {
			for ind in 0..<movesA.count {
				sc?.xStatusSetup[ind] = true
			}
		}
		
		for i in 1..<4 {
			sc?.gStatusSetup[i] = false
		}
		if movesB.count > 0 && movesB.count <= 3 {
			for ind in 1...movesB.count {
				sc?.gStatusSetup[ind] = true
			}
		}
		
//		if movesA.count == 3 && movesB.count==3 {
//			print("moves ok")
//		}
		
		if movesA.count == 3 && movesB.count==3 && padA.count == 1 && padB.count == 1 {
			for mv in movesA {
				movesX.append(mv.value)
			}
			movesX[0].color = NSColor(calibratedRed: 0.10, green: 0.20, blue: 0.85, alpha: 1.0)
			movesX[1].color = NSColor(calibratedRed: 0.85, green: 0.20, blue: 0.60, alpha: 1.0)
			movesX[2].color = NSColor(calibratedRed: 0.28, green: 0.85, blue: 0.00, alpha: 1.0)
			
			for mv in movesB {
				movesG.append(mv.value)
			}
			movesG[0].color = NSColor(calibratedRed: 0.10, green: 0.20, blue: 0.85, alpha: 1.0)
			movesG[1].color = NSColor(calibratedRed: 0.85, green: 0.20, blue: 0.60, alpha: 1.0)
			movesG[2].color = NSColor(calibratedRed: 0.28, green: 0.85, blue: 0.00, alpha: 1.0)
			
			gamepadX = padA.first!.value
			gamepadG = padB.first!.value
			
			gameStatus = .intro
			gameView.showIntro()
			startTime = time
		}
		
		////////
//		if(Move.moves.count == 6 && Gamepad.controllers.count == 2) {
//			movesX.append(Move.moves[.Player1]!)
//			movesX.append(Move.moves[.Player2]!)
//			movesX.append(Move.moves[.Player3]!)
//			movesX[0].color = NSColor(calibratedRed: 0.10, green: 0.20, blue: 0.85, alpha: 1.0)
//			movesX[1].color = NSColor(calibratedRed: 0.85, green: 0.20, blue: 0.60, alpha: 1.0)
//			movesX[2].color = NSColor(calibratedRed: 0.28, green: 0.85, blue: 0.00, alpha: 1.0)
//			movesG.append(Move.moves[.Player4]!)
//			movesG.append(Move.moves[.Player5]!)
//			movesG.append(Move.moves[.Player6]!)
//			movesG[0].color = NSColor(calibratedRed: 0.10, green: 0.20, blue: 0.85, alpha: 1.0)
//			movesG[1].color = NSColor(calibratedRed: 0.85, green: 0.20, blue: 0.60, alpha: 1.0)
//			movesG[2].color = NSColor(calibratedRed: 0.28, green: 0.85, blue: 0.00, alpha: 1.0)
//			
//			gamepadX = Gamepad.controllers[.Player1]!
//			gamepadG = Gamepad.controllers[.Player2]!
//			
//			gameStatus = .intro
//			gameView.showIntro()
//			startTime = time
//		}
	}
	
	func introAnimation(_ time: TimeInterval) {
		if gameView!.scene!.rootNode.audioPlayers.count == 0 {
			gameView!.scene!.rootNode.addAudioPlayer(SCNAudioPlayer(source: SCNAudioSource(name: "intro_3.mp3", volume: 1.0, positional: false, loops: false, shouldStream: true)))
		}
		let elapsedTime = time-startTime
		switch elapsedTime {
		case 0..<8: gameView.introFrame = 0
		case 8..<18: gameView.introFrame = 1
		case 18..<28: gameView.introFrame = 2
		case 28..<38: gameView.introFrame = 3
		case 38..<100:
			gameStatus = .phaseOne
			gameView.showPhaseOne()
			gameView!.scene!.rootNode.removeAllAudioPlayers()
			startTime = time
		default: break
		}
//		SCNTransaction.begin()
//		SCNTransaction.completionBlock = {
//			self.gameStatus = .phaseOne
//			self.gameView.showPhaseOne()
//			for mv in self.movesX {
//				mv.lightOn = false
//			}
//			for mv in self.movesG {
//				mv.lightOn = false
//			}
//			self.startTime = time
//		}
//		SCNAction.wait(duration: 10)
//		SCNTransaction.commit()
	}
	
	private var movesX = [Move]()
	private var movesG = [Move]()
	private var gamepadX: Gamepad? = nil
	private var gamepadG: Gamepad? = nil
	var actionsX:[Action] = [Action(.a, 1.5, 1.5), Action(.rt, 3.5, 1.5), Action(.lt, 5.5, 1.5), Action(.a, 7.5, 1.5), Action(.a, 9.5, 1.5), Action(.rt, 11.5, 1.5), Action(.lt, 13.5, 1.5),Action(.rt, 16.5, 1.5),Action(.lt, 19.5, 1.5),Action(.lt, 21.5, 1.5),Action(.rt, 23.5, 1.5),Action(.a, 25.5, 1.5)]
	
	var actionsG = [Action(.a, 1.5, 1.5), Action(.rt, 3.5, 1.5), Action(.lt, 5.5, 1.5), Action(.a, 7.5, 1.5), Action(.a, 9.5, 1.5), Action(.rt, 11.5, 1.5), Action(.lt, 13.5, 1.5),Action(.rt, 16.5, 1.5),Action(.lt, 19.5, 1.5),Action(.lt, 21.5, 1.5),Action(.rt, 23.5, 1.5),Action(.a, 25.5, 1.5)]
	
	func phaseOne(_ time: TimeInterval) {
//		print("\(time-startTime)")
		if gameView!.scene!.rootNode.audioPlayers.count == 0 {
			gameView!.scene!.rootNode.addAudioPlayer(SCNAudioPlayer(source: SCNAudioSource(name: "Loop_F1.mp3", volume: 1.0, positional: false, loops: true, shouldStream: true)))
		}
		let elapsedTime = time-startTime
		gameView.gammaPosition = CGPoint(x: 1280 - CGFloat(1280*CGFloat(elapsedTime/24)), y: gameView.gammaPosition.y)
		gameView.icsPosition = CGPoint(x: CGFloat(1280*CGFloat(elapsedTime/24)), y: gameView.icsPosition.y)
//		print(elapsedTime)
		var noAction = true
		for action in actionsX {
			if action.check(currentTime: elapsedTime) {
				noAction = false
				gameView.actionName = action.press.rawValue
				switch action.press {
				case .a: movesX[0].colorOn()
				case .rt: movesX[1].colorOn()
				case .lt: movesX[2].colorOn()
				default: break
				}
				
				gameView.pointX += action.check(valueA: gamepadX!.snapshot.buttonA)
				gameView.pointX += action.check(valueLt: gamepadX!.snapshot.leftTrigger)
				gameView.pointX += action.check(valueRt: gamepadX!.snapshot.rightTrigger)
				
				let m0 = movesX[0].snapshot.accelerometer
				let m1 = movesX[1].snapshot.accelerometer
				let m2 = movesX[2].snapshot.accelerometer
				gameView.pointX += action.check(value0: length(m0))
				gameView.pointX += action.check(value1: length(m1))
				gameView.pointX += action.check(value2: length(m2))
			}
		}
		for action in actionsG {
			if action.check(currentTime: elapsedTime) {
				switch action.press {
				case .a: movesG[0].colorOn()
				case .rt: movesG[1].colorOn()
				case .lt: movesG[2].colorOn()
				default: break
				}
				
				gameView.pointG += action.check(valueA: gamepadG!.snapshot.buttonA)
				gameView.pointG += action.check(valueLt: gamepadG!.snapshot.leftTrigger)
				gameView.pointG += action.check(valueRt: gamepadG!.snapshot.rightTrigger)
				
				let m0 = movesG[0].snapshot.accelerometer
				let m1 = movesG[1].snapshot.accelerometer
				let m2 = movesG[2].snapshot.accelerometer
				gameView.pointG += action.check(value0: length(m0))
				gameView.pointG += action.check(value1: length(m1))
				gameView.pointG += action.check(value2: length(m2))
			}
		}
		if noAction {
			gameView.actionName = ""
			movesX[0].colorOff()
			movesX[1].colorOff()
			movesX[2].colorOff()
			movesG[0].colorOff()
			movesG[1].colorOff()
			movesG[2].colorOff()
		}
		if elapsedTime > 30 {
			gameView!.scene!.rootNode.removeAllAudioPlayers()
			startTime = time
			gameStatus = .phaseFree
			gameView.showPhaseFree()
		}
	}
	
	var actionX0: Action? = nil
	var actionX1: Action? = nil
	var actionX2: Action? = nil
	var actionG0: Action? = nil
	var actionG1: Action? = nil
	var actionG2: Action? = nil
	
	
	func phaseFree(_ time: TimeInterval) {
		let elapsedTime = time-startTime
		let pippoPoint = 10
		let goofyTime = 0.5
		
		if gameView!.scene!.rootNode.audioPlayers.count == 0 {
			gameView!.scene!.rootNode.addAudioPlayer(SCNAudioPlayer(source: SCNAudioSource(name: "Loop_F2.mp3", volume: 1.0, positional: false, loops: true, shouldStream: true)))
		}
		
		if let action = actionX0 {
			if !action.check(currentTime: elapsedTime) {
				if !action.move0 { // didnt perform last action
					gameView.pointX -= pippoPoint
					actionX0 = nil
				}
				if gamepadG!.snapshot.buttonA > 0.5 {
					actionX0 = Action(.a, elapsedTime, goofyTime)
				}
				movesX[0].colorOff()
			} else {
				if action.press == .a {
					movesX[0].colorOn()
				} else if action.press == .wrong {
					movesX[0].wrongColor()
				} else {
					movesX[0].colorOff()
				}
				let m0 = movesX[0].snapshot.accelerometer
				let value = action.check(value0: length(m0))
				if value < 0 {
					gameView.pointX -= pippoPoint
				}
			}
		} else if actionX0 == nil {
			if gamepadG!.snapshot.buttonA > 0.5 {
				actionX0 = Action(.a, elapsedTime, goofyTime)
			}
		}
		
		if let action = actionX1 {
			if !action.check(currentTime: elapsedTime) {
				if !action.move1 { // didnt perform last action
					gameView.pointX -= pippoPoint
					actionX1 = nil
				}
				if gamepadG!.snapshot.rightTrigger > 0.5 {
					actionX1 = Action(.rt, elapsedTime, goofyTime)
				}
				movesX[1].colorOff()
			} else {
				if action.press == .rt {
					movesX[1].colorOn()
				} else if action.press == .wrong {
					movesX[1].wrongColor()
				} else {
					movesX[1].colorOff()
				}
				let m1 = movesX[1].snapshot.accelerometer
				let value = action.check(value1: length(m1))
				if value < 0 {
					gameView.pointX -= pippoPoint
				}
			}
		} else if actionX1 == nil {
			if gamepadG!.snapshot.rightTrigger > 0.5 {
				actionX1 = Action(.rt, elapsedTime, goofyTime)
			}
		}
		
		if let action = actionX2 {
			if !action.check(currentTime: elapsedTime) {
				if !action.move2 { // didnt perform last action
					gameView.pointX -= pippoPoint
					actionX2 = nil
				}
				if gamepadG!.snapshot.leftTrigger > 0.5 {
					actionX2 = Action(.lt, elapsedTime, goofyTime)
				}
				movesX[2].colorOff()
			} else {
				if action.press == .lt {
					movesX[2].colorOn()
				} else if action.press == .wrong {
					movesX[2].wrongColor()
				} else {
					movesX[2].colorOff()
				}
				let m2 = movesX[2].snapshot.accelerometer
				let value = action.check(value1: length(m2))
				if value < 0 {
					gameView.pointX -= pippoPoint
				}
			}
		} else if actionX2 == nil {
			if gamepadG!.snapshot.leftTrigger > 0.5 {
				actionX2 = Action(.lt, elapsedTime, goofyTime)
			}
		}
		
		
		
		if let action = actionG0 {
			if !action.check(currentTime: elapsedTime) {
				if !action.move0 { // didnt perform last action
					gameView.pointG -= pippoPoint
					actionG0 = nil
				}
				if gamepadX!.snapshot.buttonA > 0.5 {
					actionG0 = Action(.a, elapsedTime, goofyTime)
				}
				movesG[0].colorOff()
			} else {
				if action.press == .a {
					movesG[0].colorOn()
				} else if action.press == .wrong {
					movesG[0].wrongColor()
				} else {
					movesG[0].colorOff()
				}
				let m0 = movesG[0].snapshot.accelerometer
				let value = action.check(value0: length(m0))
				if value < 0 {
					gameView.pointG -= pippoPoint
				}
			}
		} else if actionG0 == nil {
			if gamepadX!.snapshot.buttonA > 0.5 {
				actionG0 = Action(.a, elapsedTime, goofyTime)
			}
		}
		
		if let action = actionG1 {
			if !action.check(currentTime: elapsedTime) {
				if !action.move1 { // didnt perform last action
					gameView.pointG -= pippoPoint
					actionG1 = nil
				}
				if gamepadX!.snapshot.rightTrigger > 0.5 {
					actionG1 = Action(.rt, elapsedTime, goofyTime)
				}
				movesG[1].colorOff()
			} else {
				if action.press == .rt {
					movesG[1].colorOn()
				} else if action.press == .wrong {
					movesG[1].wrongColor()
				} else {
					movesG[1].colorOff()
				}
				let m1 = movesG[1].snapshot.accelerometer
				let value = action.check(value1: length(m1))
				if value < 0 {
					gameView.pointG -= pippoPoint
				}
			}
		} else if actionG1 == nil {
			if gamepadX!.snapshot.rightTrigger > 0.5 {
				actionG1 = Action(.rt, elapsedTime, goofyTime)
			}
		}
		
		if let action = actionG2 {
			if !action.check(currentTime: elapsedTime) {
				if !action.move2 { // didnt perform last action
					gameView.pointG -= pippoPoint
					actionG2 = nil
				}
				if gamepadX!.snapshot.leftTrigger > 0.5 {
					actionG2 = Action(.lt, elapsedTime, goofyTime)
				}
				movesG[2].colorOff()
			} else {
				if action.press == .lt {
					movesG[2].colorOn()
				} else if action.press == .wrong {
					movesG[2].wrongColor()
				} else {
					movesG[2].colorOff()
				}
				let m2 = movesG[2].snapshot.accelerometer
				let value = action.check(value1: length(m2))
				if value < 0 {
					gameView.pointG -= pippoPoint
				}
			}
		} else if actionG2 == nil {
			if gamepadX!.snapshot.leftTrigger > 0.5 {
				actionG2 = Action(.lt, elapsedTime, goofyTime)
			}
		}
		
		if elapsedTime > 10 {
			if actionX0 == nil {
				if Float.random > 0.999 {
					actionX0 = Action(.wrong, elapsedTime, goofyTime)
				}
			}
			if actionX1 == nil {
				if Float.random > 0.999 {
					actionX1 = Action(.wrong, elapsedTime, goofyTime)
				}
			}
			if actionX2 == nil {
				if Float.random > 0.999 {
					actionX2 = Action(.wrong, elapsedTime, goofyTime)
				}
			}
			if actionG0 == nil {
				if Float.random > 0.999 {
					actionG0 = Action(.wrong, elapsedTime, goofyTime)
				}
			}
			if actionG1 == nil {
				if Float.random > 0.999 {
					actionG1 = Action(.wrong, elapsedTime, goofyTime)
				}
			}
			if actionG2 == nil {
				if Float.random > 0.999 {
					actionG2 = Action(.wrong, elapsedTime, goofyTime)
				}
			}
			
		}
		
		if gameView.pointX <= 0 {
			gameView.gammaWin = true
			startTime = time
			gameStatus = .phaseResult
			gameView.showPhaseResult()
		} else if gameView.pointG <= 0 {
			gameView.icsWin = true
			startTime = time
			gameStatus = .phaseResult
			gameView.showPhaseResult()
		}
	}
	
	func phaseResult(_ time: TimeInterval) {
		let elapsedTime = time-startTime
		switch elapsedTime {
		case 0..<10: break
		case 10..<100:
			gameStatus = .phaseEnd
			gameView!.scene!.rootNode.removeAllAudioPlayers()
			gameView.showEnd()
			startTime = time
		default: break
		}
	}
	
	func phaseEnd(_ time: TimeInterval) {
		let elapsedTime = time-startTime
		if gameView!.scene!.rootNode.audioPlayers.count == 0 {
			gameView!.scene!.rootNode.addAudioPlayer(SCNAudioPlayer(source: SCNAudioSource(name: "intro_4.mp3", volume: 1.0, positional: false, loops: false, shouldStream: true)))
		}
		switch elapsedTime {
		case 0..<4: gameView.endFrame = 0
		case 4..<14: gameView.endFrame = 1
		case 14..<100:
			gameStatus = .restart
			gameView!.scene!.rootNode.removeAllAudioPlayers()
			gameView.restart()
			startTime = time
		default: break
		}
	}
	func restart() {
		for move in Move.moves {
			move.value.color = NSColor(calibratedRed: 1.0, green: 1.0, blue: 1.0, alpha: 1.0)
			move.value.lightOn = true
		}
		
		movesX = []
		movesG = []
		gamepadX = nil
		gamepadG = nil
		actionsX = [Action(.a, 1.5, 1.5), Action(.rt, 3.5, 1.5), Action(.lt, 5.5, 1.5), Action(.a, 7.5, 1.5), Action(.a, 9.5, 1.5), Action(.rt, 11.5, 1.5), Action(.lt, 13.5, 1.5),Action(.rt, 16.5, 1.5),Action(.lt, 19.5, 1.5),Action(.lt, 21.5, 1.5),Action(.rt, 23.5, 1.5),Action(.a, 25.5, 1.5)]
		
		actionsG = [Action(.a, 1.5, 1.5), Action(.rt, 3.5, 1.5), Action(.lt, 5.5, 1.5), Action(.a, 7.5, 1.5), Action(.a, 9.5, 1.5), Action(.rt, 11.5, 1.5), Action(.lt, 13.5, 1.5),Action(.rt, 16.5, 1.5),Action(.lt, 19.5, 1.5),Action(.lt, 21.5, 1.5),Action(.rt, 23.5, 1.5),Action(.a, 25.5, 1.5)]
		
		actionX0 = nil
		actionX1 = nil
		actionX2 = nil
		actionG0 = nil
		actionG1 = nil
		actionG2 = nil
		
		gameStatus = .setup
		gameView.showSetupScreen()
	}
}

extension GameViewController: SCNSceneRendererDelegate {
	func renderer(_ renderer: SCNSceneRenderer, updateAtTime time: TimeInterval) {
		switch gameStatus {
		case .setup: setupPlayers(time)
		case .intro: introAnimation(time)
		case .phaseOne: phaseOne(time)
		case .phaseFree: phaseFree(time)
		case .phaseResult: phaseResult(time)
		case .phaseEnd: phaseEnd(time)
		case .restart: restart()
//		default: break
		}
//		if time > spawnTime {
//			Move.moves.first?.value.poll()
//			for (_, mv) in Move.moves {
//				mv.poll()
//			}
//			spawnShape()
//			spawnTime = time + TimeInterval(Float.random(min: 0.2, max: 1.5))
//		}
//		cleanScene()
	}
}
