//
//  Move.swift
//  Quack
//
//  Created by Marco Fattorel on 20/01/2017.
//  Copyright © 2017 Marco Fattorel. All rights reserved.
//

import Foundation
import AppKit
import SceneKit

class Move {
	enum PlayerID: Int {
		case Player1 = 0
		case Player2
		case Player3
		case Player4
		case Player5
		case Player6
		case Player7
		case Player8
		case PlayerCount
	}
	static var moves = [PlayerID: Move]()
	static func initialize() {
		guard psmove_init(PSMOVE_CURRENT_VERSION) == PSMove_True else {
			print("failed to init psmove")
			return
		}
		let connectedCount = psmove_count_connected()
		print(connectedCount)
		
//		var moves = [OpaquePointer!]()
		for m in 0..<connectedCount {
			for k in 0..<PlayerID.PlayerCount.rawValue {
				if moves.index(forKey: PlayerID(rawValue: k)!) == nil {
					let move = Move.init(with: psmove_connect_by_id(m), player: PlayerID(rawValue: k)!)
					moves[PlayerID(rawValue: k)!] = move
					print(PlayerID(rawValue: k)!.hashValue, terminator: " ")
					print("connected")
					break
				}
			}
		}
		for (_, move) in moves {
			psmove_set_leds(move.device, UInt8(move.color.redComponent*255), UInt8(move.color.greenComponent*255), UInt8(move.color.blueComponent*255));
			//				psmove_set_rumble(move, 255*(i%2));
			psmove_update_leds(move.device);
			//				psmove_util_sleep_ms(10*(i%10));
		}
		
			
			
//			var x: UnsafeMutablePointer<Int32> = UnsafeMutablePointer<Int32>.allocate(capacity: 1)
//			var y: UnsafeMutablePointer<Int32> = UnsafeMutablePointer<Int32>.allocate(capacity: 1)
//			var z: UnsafeMutablePointer<Int32> = UnsafeMutablePointer<Int32>.allocate(capacity: 1)
			//			UnsafeMutablePointer(Unmanaged.passUnretained(controller).toOpaque())
//			guard psmove_poll(mv) != 0 else {
//				print("failed to poll")
//				return
//			}
//			psmove_get_accelerometer(mv, x, y, z)
//			let xx:Int32 = UnsafeMutablePointer<Int32>(x).pointee
//			let yy:Int32 = UnsafeMutablePointer<Int32>(y).pointee
//			let zz:Int32 = UnsafeMutablePointer<Int32>(z).pointee
//			//			let xx = Unmanaged<Int32>.fromOpaque(x!).takeUnretainedValue()
//			//			let yy = Unmanaged<Int32>.fromOpaque(y!).takeUnretainedValue()
//			//			let zz = Unmanaged<Int32>.fromOpaque(z!).takeUnretainedValue()
//			print(xx, terminator: " ")
//			print(yy, terminator: " ")
//			print(zz)
	}
	private let device: OpaquePointer!
	private let playerID: PlayerID
	var color = NSColor(hue: 0.0, saturation: 0.0, brightness: 1.0, alpha: 1.0)
	var defaultColor: NSColor?
	var lightOn = true
	
//	private var color: NSColor {
//		get {
//			let hue: CGFloat = (CGFloat(playerID.rawValue)/CGFloat(PlayerID.PlayerCount.rawValue-1))
//			let saturation: CGFloat = 1.0
//			let brightness: CGFloat = 1.0
//			return NSColor(hue: hue, saturation: saturation, brightness: brightness, alpha: 1.0)
//		}
//	}
	
	struct Snapshot {
		var buttonTriangle = Double()
		var buttonCircle = Double()
		var buttonCross = Double()
		var buttonSquare = Double()
		
		var buttonSelect = Double()
		var buttonStart = Double()
		
		var buttonPS = Double()
		var buttonMove = Double()
		
		var backTrigger = Double()
		
		var accelerometer = vector3(Float(0.0), Float(0.0), Float(0.0))
		var gyroscope = vector3(Float(0.0), Float(0.0), Float(0.0))
		var magnetometer = vector3(Float(0.0), Float(0.0), Float(0.0))
	}
	var snapshot = Snapshot()
	weak var timer: Timer?
	var startTime: TimeInterval = 0.0
	var shouldResetColor = false

	init(with device: OpaquePointer, player id: PlayerID) {
		self.device = device
		self.playerID = id
		timer = Timer.scheduledTimer(withTimeInterval: 0.01, repeats: true, block: { [weak self] _ in
			self?.poll()
		})
	}
	var wrongTimer: Timer? = nil
	func wrongColor() {
		defaultColor = color
		color = NSColor(calibratedRed: 1.0, green: 0.0, blue: 0.0, alpha: 1.0)
		lightOn = true
//		wrongTimer = Timer.scheduledTimer(withTimeInterval: 1.0, repeats: false, block: { [weak self] _ in
//			self?.color = (self?.defaultColor)!
//			self?.lightOn = false
//			self?.wrongTimer?.invalidate()
//			self?.wrongTimer = nil
//		})
	}
	func colorOn() {
		if let cl = defaultColor {
			color = cl
			defaultColor = nil
//			tm.invalidate()
		}
		lightOn = true
	}
	func colorOff() {
//		if let tm = wrongTimer {
//		} else {
			lightOn = false
//		}
	}
	func poll() {
		// Keep led color alive
		
		if(lightOn) {
			psmove_set_leds(device, UInt8(color.redComponent*255), UInt8(color.greenComponent*255), UInt8(color.blueComponent*255))
		} else {
			psmove_set_leds(device, 0, 0, 0)
		}
		psmove_update_leds(device);
		
		
//		guard psmove_poll(device) != 0 else {
//			print("failed to poll")
//			return
//		}
		while(psmove_poll(device) == 0) { }
		
		let buttons = psmove_get_buttons(device)
		snapshot.buttonTriangle = (buttons & Btn_TRIANGLE.rawValue) != 0 ? 1.0 : 0.0
		snapshot.buttonSquare = (buttons & Btn_SQUARE.rawValue) != 0 ? 1.0 : 0.0
		snapshot.buttonCircle = (buttons & Btn_CIRCLE.rawValue) != 0 ? 1.0 : 0.0
		snapshot.buttonCross = (buttons & Btn_CROSS.rawValue) != 0 ? 1.0 : 0.0
		
		snapshot.buttonSelect = (buttons & Btn_SELECT.rawValue) != 0 ? 1.0 : 0.0
		snapshot.buttonStart = (buttons & Btn_START.rawValue) != 0 ? 1.0 : 0.0
		
		snapshot.buttonPS = (buttons & Btn_PS.rawValue) != 0 ? 1.0 : 0.0
		snapshot.buttonMove = (buttons & Btn_MOVE.rawValue) != 0 ? 1.0 : 0.0
		
//		print(psmove_get_trigger(device))
		snapshot.backTrigger = Double(psmove_get_trigger(device))/255.0
		
		let x: UnsafeMutablePointer<Float> = UnsafeMutablePointer<Float>.allocate(capacity: 1)
		let y: UnsafeMutablePointer<Float> = UnsafeMutablePointer<Float>.allocate(capacity: 1)
		let z: UnsafeMutablePointer<Float> = UnsafeMutablePointer<Float>.allocate(capacity: 1)
		psmove_get_accelerometer_frame(device, Frame_SecondHalf, x, y, z)
		let xx = UnsafeMutablePointer<Float>(x).pointee
		let yy = UnsafeMutablePointer<Float>(y).pointee
		let zz = UnsafeMutablePointer<Float>(z).pointee
		
		snapshot.accelerometer = vector3(xx, yy, zz)
//		if playerID == .Player1 {
//			print(length(snapshot.accelerometer))
//		}
	}
	
	deinit {
		psmove_disconnect(device)
		timer?.invalidate()
	}
		
		
		
		
//		var moves = [OpaquePointer!]()
//		for m in 0..<connectedCount {
//			let mv = psmove_connect_by_id(m)
//			moves.append(mv)
//			
//			var x: UnsafeMutablePointer<Int32> = UnsafeMutablePointer<Int32>.allocate(capacity: 1)
//			var y: UnsafeMutablePointer<Int32> = UnsafeMutablePointer<Int32>.allocate(capacity: 1)
//			var z: UnsafeMutablePointer<Int32> = UnsafeMutablePointer<Int32>.allocate(capacity: 1)
////			UnsafeMutablePointer(Unmanaged.passUnretained(controller).toOpaque())
//			guard psmove_poll(mv) != 0 else {
//				print("failed to poll")
//				return
//			}
//			psmove_get_accelerometer(mv, x, y, z)
//			let xx:Int32 = UnsafeMutablePointer<Int32>(x).pointee
//			let yy:Int32 = UnsafeMutablePointer<Int32>(y).pointee
//			let zz:Int32 = UnsafeMutablePointer<Int32>(z).pointee
////			let xx = Unmanaged<Int32>.fromOpaque(x!).takeUnretainedValue()
////			let yy = Unmanaged<Int32>.fromOpaque(y!).takeUnretainedValue()
////			let zz = Unmanaged<Int32>.fromOpaque(z!).takeUnretainedValue()
//			print(xx, terminator: " ")
//			print(yy, terminator: " ")
//			print(zz)
//		}
		
		
//		var move:
}
